"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const copy_1 = require("./helpers/copy");
const NodeMutator_1 = require("./mutators/NodeMutator");
const BabelHelper_1 = require("./helpers/BabelHelper");
const plugin_1 = require("@stryker-mutator/api/plugin");
class JavaScriptMutator {
    constructor(log, mutators) {
        this.log = log;
        this.mutators = mutators;
    }
    mutate(inputFiles) {
        const mutants = [];
        inputFiles.forEach(file => {
            const ast = BabelHelper_1.default.parse(file.textContent);
            BabelHelper_1.default.getNodes(ast).forEach(node => {
                this.mutators.forEach(mutator => {
                    const mutatedNodes = mutator.mutate(node, copy_1.default);
                    if (mutatedNodes) {
                        const newMutants = this.generateMutants(mutatedNodes, mutator.name, file.name);
                        mutants.push(...newMutants);
                    }
                });
            });
        });
        return mutants;
    }
    generateMutants(mutatedNodes, mutatorName, fileName) {
        const mutants = [];
        mutatedNodes.forEach(node => {
            const replacement = BabelHelper_1.default.generateCode(node);
            if (node.start !== null && node.end !== null) {
                const range = [node.start, node.end];
                const mutant = {
                    fileName,
                    mutatorName,
                    range,
                    replacement
                };
                this.log.trace(`Generated mutant for mutator ${mutatorName} in file ${fileName} with replacement code "${replacement}"`);
                mutants.push(mutant);
            }
        });
        return mutants;
    }
}
JavaScriptMutator.inject = plugin_1.tokens(plugin_1.commonTokens.logger, NodeMutator_1.NODE_MUTATORS_TOKEN);
exports.JavaScriptMutator = JavaScriptMutator;
//# sourceMappingURL=JavaScriptMutator.js.map